unit dplay ;

interface

uses OLE2, Windows ;

(*==========================================================================;
 *
 *  Copyright (C) 1994-1995 Microsoft Corporation.  All Rights Reserved.
 *
 *  File:       dplay.h
 *  Content:    DirectPlay include file
 *
 *  DirectX 2 Delphi adaptation by Blake Stone
 *
 *    EMail:     bstone@dkw.com
 *    Home Page: http://www.dkw.com/bstone
 *
 ***************************************************************************)

const
  IID_IDirectPlay: TGUID = ( D1:$5454e9a0;D2:$db65;D3:$11ce;D4:($92,$1c,$00,$aa,$00,$6c,$49,$72)) ;

// #pragma pack(push, 1) { What, if anything, needs to be done about this? }

(*============================================================================
 *
 * DirectPlay Structures
 *
 * Various structures used to invoke DirectPlay.
 *
 *==========================================================================*)

const
  DPLONGNAMELEN           = 52 ;
  DPSHORTNAMELEN          = 20 ;
  DPSESSIONNAMELEN        = 32 ;
  DPPASSWORDLEN           = 16 ;
  DPUSERRESERVED          = 16 ;

type
  IDirectPlay = class ;

  DPID = DWORD ;
//  LPDPID = ^DPID ;

  DPCAPS = record
    dwSize: DWORD ;
    dwFlags: DWORD ;
    dwMaxBufferSize: DWORD ;
    dwMaxQueueSize: DWORD ;             // Function of DPlay, not SP.
    dwMaxPlayers: DWORD ;
    dwHundredBaud: DWORD ;              // 24 is 2400, 96 is 9600, etc.
    dwLatency: DWORD ;
  end ;
//  LPDPCAPS = ^DPCAPS ;

  DPSESSIONDESC = record
    dwSize: DWORD ;
    guidSession: TGUID ;                  // Id for Game.  Null is all games.
    dwSession: DWORD ;                    // session identifier
    dwMaxPlayers: DWORD ;                 // Maximum players allowed in game.
    dwCurrentPlayers: DWORD ;             // Current  players in Game.
    dwFlags: DWORD ;                      // DPOPEN_* flags
    szSessionName: Array [ 0..DPSESSIONNAMELEN-1 ] of char ; // Human readable name for Game
    szUserField: Array [ 0..DPUSERRESERVED-1 ] of char ;
    dwReserved1: DWORD ;                  // Reserved for future MS use.
    szPassword: Array [ 0..DPPASSWORDLEN-1 ] of char ; // Password to be allowed into game.
    dwReserved2: DWORD ;                  // Reserved for future MS use.
    dwUser1: DWORD ;
    dwUser2: DWORD ;
    dwUser3: DWORD ;
    dwUser4: DWORD ;
  end ;
//  LPDPSESSIONDESC = ^DPSESSIONDESC ;

 (* Player enumeration callback prototype *)

  LPDPENUMPLAYERSCALLBACK = function ( dpId: DPID ; lpFriendlyName: LPSTR ;
      lpFormalName: LPSTR ; dwFlags: DWORD ; lpContext: Pointer ): BOOL ;
  LPDPENUMSESSIONSCALLBACK = function ( const lpDPSGameDesc: DPSESSIONDESC ;
      lpContext: Pointer ; var lpdwTimeOut: DWORD ; dwFlags: DWORD ): BOOL ;

(*
 * IDirectPlay
 *)

  IDirectPlay = class ( IUnknown )
    (*** IDirectPlay methods ***)
    function AddPlayerToGroup ( pidGroup: DPID ; pidPlayer: DPID ): HRESULT ;
        virtual ; stdcall ; abstract ;
    function Close: HRESULT ; virtual ; stdcall ; abstract ;
    function CreatePlayer ( var lppidID: DPID ; lpPlayerFriendlyName: LPSTR ;
        lpPlayerFormalName: LPSTR ; lpEvent: PHandle ): HRESULT ; virtual ;
            stdcall ; abstract ;
    function CreateGroup ( var lppidID: DPID ; lpGroupFriendlyName: LPSTR ;
        lpGroupFormalName: LPSTR ): HRESULT ; virtual ; stdcall ; abstract ;
    function DeletePlayerFromGroup ( pidGroup: DPID ; pidPlayer: DPID):
        HRESULT ; virtual ; stdcall ; abstract ;
    function DestroyPlayer ( pidID: DPID ): HRESULT ; virtual ; stdcall ;
        abstract ;
    function DestroyGroup ( pidID: DPID ): HRESULT ; virtual ; stdcall ;
        abstract ;
    function EnableNewPlayers ( bEnable: BOOL ): HRESULT ; virtual ; stdcall ;
        abstract ;
    function EnumGroupPlayers ( pidGroupPID: DPID ;
        lpEnumPlayersCallback: LPDPENUMPLAYERSCALLBACK ;
        lpContext: Pointer ; dwFlags: DWORD ): HRESULT ; virtual ; stdcall ;
        abstract ;
    function EnumGroups ( dwSessionID: DWORD ;
        lpEnumPlayersCallback: LPDPENUMPLAYERSCALLBACK ; lpContext: Pointer ;
        dwFlags: DWORD ): HRESULT ; virtual ; stdcall ; abstract ;
    function EnumPlayers ( dwSessionId: DWORD ;
        lpEnumPlayersCallback: LPDPENUMPLAYERSCALLBACK ; lpContext: Pointer ;
        dwFlags: DWORD ): HRESULT ; virtual ; stdcall ; abstract ;
    function EnumSessions ( const lpSDesc: DPSESSIONDESC ; dwTimeout: DWORD ;
        lpEnumSessionsCallback: LPDPENUMSESSIONSCALLBACK ; lpContext: Pointer ;
        dwFlags: DWORD ): HRESULT ; virtual ; stdcall ; abstract ;
    function GetCaps ( const lpDPCaps: DPCAPS ): HRESULT ; virtual ; stdcall ;
        abstract ;
    function GetMessageCount ( pidID: DPID ; var lpdwCount: DWORD ): HRESULT ;
        virtual ; stdcall ; abstract ;
    function GetPlayerCaps ( pidID: DPID ; const lpDPPlayerCaps: DPCAPS ):
        HRESULT ; virtual ; stdcall ; abstract ;
    function GetPlayerName ( pidID: DPID ; lpPlayerFriendlyName: LPSTR ;
        var lpdwFriendlyNameLength: DWORD ; lpPlayerFormalName: LPSTR ;
        var lpdwFormalNameLength: DWORD ): HRESULT ; virtual ; stdcall ;
        abstract ;
    function Initialize ( lpGUID: PGUID ): HRESULT ; virtual ; stdcall ;
        abstract ;
    function Open ( const lpSDesc: DPSESSIONDESC ): HRESULT ; virtual ;
        stdcall ; abstract ;
    function Receive ( var lppidFrom, lppidTo: DPID ; dwFlags: DWORD ;
        lpvBuffer: Pointer ; var lpdwSize: DWORD ): HRESULT ; virtual ;
        stdcall ; abstract ;
    (*** Warning!  SaveSession is defined differently in the header files
         and the online documentation ***)
    function SaveSession ( lpSessionName: LPSTR ): HRESULT ; virtual ;
        stdcall ; abstract ;
    function Send ( pidFrom: DPID ; pidTo: DPID ; dwFlags: DWORD ;
        lpvBuffer: Pointer ; dwBuffSize: DWORD ): HRESULT ; virtual ; stdcall ;
        abstract ;
    function SetPlayerName ( pidID: DPID ; lpPlayerFriendlyName: LPSTR ;
        lpPlayerFormalName: LPSTR ): HRESULT ; virtual ; stdcall ; abstract ;
  end ;

(****************************************************************************
 *
 * DIRECTPLAY ERRORS
 *
 * Errors are represented by negative values and cannot be combined.
 *
 ****************************************************************************)

const
  DP_OK                           = 0 ;
  DPERR_ALREADYINITIALIZED        = $88770000 + 5 ;
  DPERR_ACCESSDENIED              = $88770000 + 10 ;
  DPERR_ACTIVEPLAYERS             = $88770000 + 20 ;
  DPERR_BUFFERTOOSMALL            = $88770000 + 30 ;
  DPERR_CANTADDPLAYER             = $88770000 + 40 ;
  DPERR_CANTCREATEGROUP           = $88770000 + 50 ;
  DPERR_CANTCREATEPLAYER          = $88770000 + 60 ;
  DPERR_CANTCREATESESSION         = $88770000 + 70 ;
  DPERR_CAPSNOTAVAILABLEYET       = $88770000 + 80 ;
  DPERR_EXCEPTION                 = $88770000 + 90 ;
  DPERR_GENERIC                   = E_FAIL ;

  DPERR_INVALIDFLAGS              = $88770000 + 120 ;
  DPERR_INVALIDOBJECT             = $88770000 + 130 ;
  DPERR_INVALIDPARAM              = E_INVALIDARG ;
  DPERR_INVALIDPARAMS             = DPERR_INVALIDPARAM ;
  DPERR_INVALIDPLAYER             = $88770000 + 150 ;
  DPERR_NOCAPS                    = $88770000 + 160 ;
  DPERR_NOCONNECTION              = $88770000 + 170 ;
  DPERR_NOMEMORY                  = E_OUTOFMEMORY ;
  DPERR_OUTOFMEMORY               = DPERR_NOMEMORY ;
  DPERR_NOMESSAGES                = $88770000 + 190 ;
  DPERR_NONAMESERVERFOUND         = $88770000 + 200 ;
  DPERR_NOPLAYERS                 = $88770000 + 210 ;
  DPERR_NOSESSIONS                = $88770000 + 220 ;
  DPERR_SENDTOOBIG                = $88770000 + 230 ;
  DPERR_TIMEOUT                   = $88770000 + 240 ;
  DPERR_UNAVAILABLE               = $88770000 + 250 ;
  DPERR_UNSUPPORTED               = E_NOTIMPL ;
  DPERR_BUSY                      = $88770000 + 270 ;
  DPERR_USERCANCEL                = $88770000 + 280 ;

  DPOPEN_OPENSESSION          = $00000001 ;
  DPOPEN_CREATESESSION        = $00000002 ;

  DPSEND_GUARANTEE            = $00000001 ;
  DPSEND_HIGHPRIORITY         = $00000002 ;
  DPSEND_TRYONCE              = $00000004 ;

  DPRECEIVE_ALL               = $00000001 ;
  DPRECEIVE_TOPLAYER          = $00000002 ;
  DPRECEIVE_FROMPLAYER        = $00000004 ;
  DPRECEIVE_PEEK              = $00000008 ;

  DPCAPS_NAMESERVICE          = $00000001 ; // A name server is supported.
  DPCAPS_NAMESERVER           = $00000002 ; // You are the name server.
  DPCAPS_GUARANTEED           = $00000004 ; // SP's don't have to implement guarantees.

  DPENUMSESSIONS_AVAILABLE    = $00000001 ; // All games that match password (if given)
                                            // and have openings.
  DPENUMSESSIONS_ALL          = $00000002 ;
  DPENUMSESSIONS_PREVIOUS     = $00000004 ;

  DPENUMPLAYERS_ALL           = $00000000 ;
  DPENUMPLAYERS_PREVIOUS      = $00000004 ;
  DPENUMPLAYERS_LOCAL         = $00000008 ;
  DPENUMPLAYERS_REMOTE        = $00000010 ;
  DPENUMPLAYERS_GROUP         = $00000020 ;
  DPENUMPLAYERS_SESSION       = $00000080 ;

//
// This flag is set on the enumsessions callback when the time out has occured.
// This means that there is no session data for this callback.
// If lpdwTimeOut is set to a non-zero value and the EnumSessionsCallback returns
// TRUE then EnumSessions will continue until the next timeout occurs.
// Timeouts are in milliseconds.

  DPESC_TIMEDOUT              = $00000001 ;

//
// System message structures and types.
//
// System messages have a leading 4 byte type code to identify the message.
// an app knows it is a system message because it is addressed 'To' player 0.
//


  DPSYS_ADDPLAYER               = $0003 ;  // DPMSG_ADDPLAYER
  DPSYS_DELETEPLAYER            = $0005 ;  // DPMSG_DELETEPLAYER

  DPSYS_ADDPLAYERTOGROUP        = $0007 ;  // DPMSG_GROUPADD

  DPSYS_INVITE                  = $000e ;  // DPMSG_INVITE, Net only.

  DPSYS_DELETEGROUP             = $0020 ;  // DPMSG_DELETEPLAYER
  DPSYS_DELETEPLAYERFROMGRP     = $0021 ;  // DPMSG_GROUPDELETE
  DPSYS_SESSIONLOST             = $0031 ;

  DPSYS_CONNECT                 = $484b ;  // DPMSG_GENERIC

type
  DPMSG_ADDPLAYER = record
    dwType: DWORD ;
    dwPlayerType: DWORD ;
    dpId: DPID ;
    szLongName: Array [ 0..DPLONGNAMELEN-1 ] of char ;
    szShortName: Array [ 0..DPSHORTNAMELEN-1 ] of char ;
    dwCurrentPlayers: DWORD ;
  end ;
  DPMSG_ADDGROUP = DPMSG_ADDPLAYER ;

  DPMSG_GROUPADD = record
    dwType: DWORD ;
    dpIdGroup: DPID ;
    dpIdPlayer: DPID ;
  end ;
  DPMSG_GROUPDELETE = DPMSG_GROUPADD ;

  DPMSG_DELETEPLAYER = record
    dwType: DWORD ;
    dpId: DPID ;
  end ;

  DPMSG_INVITE = record
    dwType: DWORD ;
    dpsDesc: DPSESSIONDESC ;
  end ;

  DPMSG_GENERIC = record
    dwType: DWORD ;
  end ;

(*
 * Create API
 *)

  LPDPENUMDPCALLBACK = function ( lpSPGuid: PGUID ; lpFriendlyName: LPSTR ;
      dwMajorVersion: DWORD ; dwMinorVersion: DWORD ; lpContext: Pointer ):BOOL ;

function DirectPlayCreate ( lpGUID: PGUID ; var lplpDP: IDirectPlay ;
    pUnk: IUnknown ): HRESULT ; stdcall ;
function DirectPlayEnumerate ( lpEnumDPCallback: LPDPENUMDPCALLBACK ;
    lpContext: Pointer ): HRESULT ; stdcall ;

// #pragma pack(pop)

implementation

function DirectPlayCreate ; external 'DPLAY.DLL' ;
function DirectPlayEnumerate ; external 'DPLAY.DLL' ;

end.

