unit DirectDraw;

interface

uses Forms, SysUtils, DDraw, OLE2, Windows ;

type
  TDDScreen = class ( TObject )
  private
    FFrontSurface, FBackSurface: IDirectDrawSurface ;
  public
    constructor Create ( Width, Height, Depth: DWORD ) ;
    destructor Destroy ; override ;
    procedure Flip ;
    property FrontSurface: IDirectDrawSurface read FFrontSurface ;
    property BackSurface: IDirectDrawSurface read FBackSurface ;
  end ;

implementation

var
  DDraw: IDirectDraw ;
  DDResult: HRESULT ;
  DDPalette: IDirectDrawPalette ;

{-----------------------------------------------------------
 |
 | TDDSCREEN.CREATE
 |
 | Set up DirectDrawSurface objects
 |
 -----------------------------------------------------------}

constructor TDDScreen.Create ( Width, Height, Depth: DWORD ) ;
const
  SurfaceDesc: DDSURFACEDESC = (
      dwSize: sizeof ( SurfaceDesc ) ;
      dwFlags: DDSD_CAPS + DDSD_BACKBUFFERCOUNT ;
      dwBackBufferCount: 2 ;
      ddsCaps: ( dwCaps: DDSCAPS_PRIMARYSURFACE + DDSCAPS_FLIP + DDSCAPS_COMPLEX )
      ) ;
var
  BackCaps: DDSCAPS ;
  Index: Integer ;
  PEntry: Array [ 0..255 ] of TPaletteEntry ;
begin
  inherited Create ;

  { Initialize the DirectDraw system }

  DDResult := DirectDrawCreate ( nil, DDraw, nil ) ;
  if DDResult <> DD_OK then
    raise Exception.Create (
        Format ( 'DirectDrawCreate failed: %x', [ DDResult ] ) ) ;

  { Set no cooperation with other applications }

  DDResult := DDraw.SetCooperativeLevel ( Application.Handle,
      DDSCL_FULLSCREEN + DDSCL_ALLOWREBOOT + DDSCL_NOWINDOWCHANGES +
      DDSCL_ALLOWMODEX + DDSCL_EXCLUSIVE ) ;
  if DDResult <> DD_OK then
    raise Exception.Create ( Format (
        'IDirectSound.SetCooperativeLevel failed: %x', [ DDResult ] ) ) ;

  { Set display mode }

  DDResult := DDraw.SetDisplayMode ( Width, Height, Depth ) ;
  if DDResult <> DD_OK then
    raise Exception.Create ( Format (
        'IDirectDraw.SetDisplayMode failed: %x', [ DDResult ] ) ) ;

  { Create triple-buffered backing store }

  DDResult := DDraw.CreateSurface ( SurfaceDesc, FFrontSurface, nil ) ;
  if DDResult <> DD_OK then
    raise Exception.Create ( Format (
        'IDirectDraw.CreateSurface failed: %x', [ DDResult ] ) ) ;

  BackCaps.dwCaps := DDSCAPS_BACKBUFFER ;
  DDResult := FFrontSurface.GetAttachedSurface ( BackCaps, FBackSurface ) ;
  if DDResult <> DD_OK then
    raise Exception.Create ( Format (
        'IDirectDrawSurface.GetAttachedSurface failed: %x', [ DDResult ] ) ) ;

  { Create a palette }

  for Index := 0 to 255 do
  with PEntry [ Index ] do
  begin
    peRed := Random ( 255 ) ;
    peBlue := Random ( 255 ) ;
    peGreen := Random ( 255 ) ;
    peFlags := 0 ;
  end ;
  DDraw.CreatePalette ( DDPCAPS_8BIT, @PEntry[0], DDPalette, nil ) ;
  FFrontSurface.SetPalette ( DDPalette ) ;
end ;

{-----------------------------------------------------------
 |
 | TDDSCREEN.DESTROY
 |
 | Release DirectDrawSurface objects
 |
 -----------------------------------------------------------}

destructor TDDScreen.Destroy ;
begin
  if FFrontSurface <> nil then
    FFrontSurface.Release ;
  FFrontSurface := nil ;
  if DDraw <> nil then
    DDraw.Release ;
  DDraw := nil ;
  inherited Destroy ;
end ;

{-----------------------------------------------------------
 |
 | TDDSCREEN.FLIP
 |
 | Flip to a new display buffer
 |
 -----------------------------------------------------------}

procedure TDDScreen.Flip ;
begin
  FFrontSurface.Flip ( nil, DDFLIP_WAIT ) ;
end ;

{-----------------------------------------------------------
 |
 | INITIALIZATION
 |
 | << ? >>
 |
 -----------------------------------------------------------}

initialization
begin
end ;

{-----------------------------------------------------------
 |
 | FINALIZATION
 |
 | << ? >>
 |
 -----------------------------------------------------------}

finalization
begin

end ;

end.
